
#include "MedalWinner.h"

/**
 * Replaces/sets the won medals of a competitor.
 *
 * @param medals The won medals with their amount.
 * @return True, if successful.
 */
bool MedalWinner::setMedals(const QJsonObject &medals) {
    if (!medals.contains("ME_GOLD")
        || !medals.contains("ME_SILVER")
        || !medals.contains("ME_BRONZE")) {
        return false;
    }

    setGold(medals["ME_GOLD"].toInt());
    setSilver(medals["ME_SILVER"].toInt());
    setBronze(medals["ME_BRONZE"].toInt());

    return true;
}

/**
 * Static compare method, which can compare the amount of medals of two MedalWinners.
 * Gold has the highest priority, then m_silver and finally m_bronze.
 *
 * @param lComp First competitor to compare.
 * @param rComp Second competitor to compare.
 * @return True, if the second competitor got more or higher medals.
 */
bool MedalWinner::compare(MedalWinner lComp, MedalWinner rComp) {
    // create difference between medal amounts
    int gold = lComp.getGold() - rComp.getGold();
    int silver = lComp.getSilver() - rComp.getSilver();
    int bronze = lComp.getBronze() - rComp.getBronze();

    // compare medal differences
    return gold < 0 || (gold == 0 && (silver < 0 || (silver == 0 && bronze < 0)));
}

bool MedalWinner::setMedalWinner(const QJsonObject &competitor) {
    setCompetitor(competitor);

    if (!competitor.contains("medals")) return false;
    QJsonObject medals = competitor["medals"].toObject();
    setMedals(medals);
    return true;
}

void MedalWinner::setMedalWinner(const MedalWinner &competitor) {
    setCompetitor(competitor);

    setGold(competitor.m_gold);
    setSilver(competitor.m_silver);
    setBronze(competitor.m_bronze);
}
